#define std_lang_Object_IMPORT
#define std_util_ArraySet_IMPORT

#include <stdlib.h>
#include <std/util/impl/ArraySet.h>

union ArraySet
ArraySet_create(void)
{
    struct impl_ArraySet *impl;

    if ((impl = (struct impl_ArraySet *)malloc(sizeof(*impl))) == NULL) {
	return (union ArraySet)Object_NULL;
    }
    impl->data = NULL;
    impl->size = 0;
    impl->capacity = 0;
    return (union ArraySet)impl;
}

void
ArraySet_free(union ArraySet set)
{
    struct impl_ArraySet *impl = set.impl;

    if (impl == NULL)
	return;
    free(impl->data);
    free(impl);
}

size_t
ArraySet_getSize(union ArraySet set)
{
    struct impl_ArraySet *impl = set.impl;

    return impl->size;
}

void *
ArraySet_get(union ArraySet set, unsigned int k)
{
    struct impl_ArraySet *impl = set.impl;

    if (k >= impl->size)
	return NULL;
    return impl->data[k];
}

int
ArraySet_add(union ArraySet set, void *e)
{
    static const size_t DELTA = 4;
    struct impl_ArraySet *impl = set.impl;
    struct impl_ArraySet next;

    next.size = impl->size + 1;
    if (impl->capacity < next.size) {
        next.capacity = next.size + DELTA;
        if ((next.data = realloc(impl->data,
				 next.capacity * sizeof(e))) == NULL) {
            return -1;
        }
        impl->data = next.data;
        impl->capacity = next.capacity;
    }
    impl->data[impl->size] = e;
    impl->size = next.size;
    return 0;
}

int
ArraySet_remove(union ArraySet set, void *e)
{
    struct impl_ArraySet *impl = set.impl;
    unsigned int k;

    if (impl->size == 0)
	return -1;
    for (k = 0; k < impl->size && impl->data[k] != e; ++k)
	;
    if (k == impl->size)
	return -1;
    if (--(impl->size) == 0) {
	free(impl->data);
	impl->data = NULL;
	impl->capacity = 0;
	return 0;
    }
    impl->data[k] = impl->data[impl->size];
    return 0;
}
