#define std_lang_Object_IMPORT
#define std_util_LinkedList_IMPORT

#include <stdlib.h>
#include <std/util/impl/LinkedList.h>

static struct LinkedList_Node *
LinkedList_Node_create(void *value)
{
    struct LinkedList_Node *node;

    if ((node = (struct LinkedList_Node *)malloc(sizeof(*node))) == NULL)
        return NULL;
    node->next = NULL;
    node->value = value;
    return node;
}

static void
LinkedList_Node_free(struct LinkedList_Node *node)
{
    free(node);
}

union LinkedList
LinkedList_create(void)
{
    struct impl_LinkedList *impl;

    if ((impl = (struct impl_LinkedList *)malloc(sizeof(*impl))) == NULL) {
	return (union LinkedList)Object_NULL;
    }
    impl->head = NULL;
    impl->next = &impl->head;
    return (union LinkedList)impl;
}

void
LinkedList_free(union LinkedList list)
{
    struct impl_LinkedList *impl = list.impl;
    struct LinkedList_Node *node = impl->head, *next;

    while (node != NULL) {
	next = node->next;
	LinkedList_Node_free(node);
	node = next;
    }
    free(impl);
}

int
LinkedList_append(union LinkedList list, void *e)
{
    struct impl_LinkedList *impl = list.impl;
    struct LinkedList_Node *node;

    if ((node = LinkedList_Node_create(e)) == NULL) {
	return -1;
    }
    *(impl->next) = node;
    impl->next = &node->next;
    return 0;
}

void *
LinkedList_getFirst(union LinkedList list)
{
    struct impl_LinkedList *impl = list.impl;
    struct LinkedList_Node *node = impl->head;
    void *e;

    if (node == NULL)
	return NULL;
    if ((impl->head = node->next) == NULL) {
	impl->next = &impl->head;
    }
    e = node->value;
    LinkedList_Node_free(node);
    return e;
}

struct LinkedList_Node *
LinkedList_getHeadNode(union LinkedList list)
{
    struct impl_LinkedList *impl = list.impl;

    return impl->head;
}
